#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.

"""
Looks up contact authors for a given project.

If this tool isn't working, the same information can be obtained by logging
into the OPT, opening the project in question, and clicking on the coauthors.

Originally imported from https://svn.aoc.nrao.edu/repos/SSS/ssa-libraries/trunk/sss-dao/src/main/sh/project-contacts.py

DKL moved it from his local filesystem into the repo at that location, so it is safe to assume that it is not
under active development at that location.

Usage:

"""
import argparse
import json
import logging
import sys

# pylint: disable=C0103, C0301, E0401, R0903
from typing import List

import dsnparse
import MySQLdb
import psycopg2
from pycapo import CapoConfig

logger = logging.getLogger("contacts_wrest")
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))


class ProjectContactsWrester:
    """Retrieves contact email addresses from the OPT and PST database for a given project"""

    def __init__(self, project_code: str):
        self.project_code = project_code

    def retrieve_contacts(self) -> List[str]:
        """
        Get the designated contact(s) for this project.

        :return: email addresses of contacts
        """

        # step 1: retrieve the contact author globalIDs from the OPT
        global_ids = self._get_global_ids()

        # step 2: look up email addresses in the my.nrao.edu database
        return self._get_emails_for_global_ids(global_ids)

    def _get_global_ids(self):
        """
        We'll need the global IDs for this project

        :return:
        """
        query = """select globalid
from project
join projectauthor p on project.pi = p.id
where projectcode = %(project_code)s and receivesemail
union
select globalid
from project
join projectauthor p on project.contactauthor = p.id
where projectcode = %(project_code)s and receivesemail
union
select globalid
from project
join coauthors c on project.id = c.project_id
join projectauthor p on c.projectauthor_id = p.id
where projectcode = %(project_code)s and receivesemail"""

        config = CapoConfig().settings("edu.nrao.ssa.project.db")
        dsn = dsnparse.parse(config["url"])
        with psycopg2.connect(
            host=dsn.host, dbname=dsn.paths[0], user=config["username"], password=config["password"]
        ) as conn:
            cur = conn.cursor()
            cur.execute(query, {"project_code": self.project_code})
            return [x[0] for x in cur.fetchall()]

    def _get_emails_for_global_ids(self, globalids: List[int]) -> List[str]:
        """
        Now that we have the global IDs, we can use them to look up contacts' email addresses

        :param globalids: global IDs of designated project contacts
        :return:
        """
        query = """select email
from email
where person_id IN (%s) and defaultEmail"""

        format_strings = ",".join(["%s"] * len(globalids))
        config = CapoConfig().settings("my.nrao.jdbc")
        dsn = dsnparse.parse(config["URL"])
        conn = MySQLdb.connect(host=dsn.host, db=dsn.paths[0], user=config["username"], password=config["password"])
        cur = conn.cursor()
        cur.execute(query % format_strings, tuple(globalids))
        result = [x[0] for x in cur.fetchall()]
        conn.close()
        return result


def main():
    """
    Retrieves contact email addresses from the OPT and PST database for a given project

    :return:
    """
    parser = argparse.ArgumentParser(description="Retrieve contact email addresses from the OPT and PST database")
    parser.add_argument("project_code", help="Project code of the project to look up the contactable authors for")
    ns = parser.parse_args()

    # Retrieve project contacts' email addresses freom my.nrao.edu
    emails = ProjectContactsWrester(ns.project_code).retrieve_contacts()

    # Display them
    json.dump(emails, sys.stdout)


if __name__ == "__main__":
    main()
