#!/bin/bash
#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.

# Set failfast
set -e
set -o pipefail

echo "Happy Testing!"

# Set path to service unit tests
path_to_test="test"

# Set skip behavior
skip_arg="--skip-empty"

# Install testing requirements
init () {
  pushd /packages/testing
  python setup.py develop --user
  popd

#  if exists, remove old .coverage file
  if [[ -e .coverage ]]; then
    echo "Removing old coverage file..."
    rm .coverage
  fi
}

run_tests () {
  # Look for lines in requirements.txt starting with "-e"
  # to find paths to packages containings tests to be executed.
  pkgs=($(sed -n "s|-e ..||p" requirements.txt | grep -v testing | tr "\n" " "))
  pkgs+=("$path_to_test")

  for pkg in "${pkgs[@]}"
  do
      cd "$pkg" || exit
      if [[ $# -eq 0 ]]; then
        pytest
      else
        pytestWithCoverage $1
      fi
      cd /code/ || exit
  done
}

pytestWithCoverage () {
  coverage run --parallel-mode --branch -m pytest
  if [ "$1" == "b" ]; then
    cd /code/ && coverage combine --rcfile="/code/bin/config/.coveragerc" --append "$pkg" && cd "$pkg"
  else
    cd /code/ && coverage combine --append "$pkg" && cd "$pkg"
  fi
}

while getopts "cbr:o:" OPTION
do
	case $OPTION in
		c)
			init
      run_tests "$OPTION"
      coverage report "$skip_arg" --omit="**/test_*.py,**/_version.py,**/conftest.py,**/*interfaces.py"
			;;
		b)
			init
      run_tests "$OPTION"
			;;
	  r)
			echo "The generating coverage report as: $OPTARG"
			report_output=$OPTARG
			;;
    o)
      echo "Naming coverage report: $OPTARG"
      report_name=$OPTARG
      ;;
		\?)
			echo "Option not recognized"
			exit
			;;
	esac
done

if [[ $# -eq 0 ]] ; then
  init
  run_tests
fi

if [[ -n $report_output  ]]; then
  if [[ -n $report_name ]]; then
    coverage "$report_output" -o $report_name.$report_output "$skip_arg"
  else
    coverage "$report_output" "$skip_arg"
  fi
fi

echo "Tests have finished."
